@@ -19,8 +19,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import os
 
-version = "0.12"
+
+version = "0.15"
 version_info = tuple([int(x) for x in version.split(".")])
 
 
@@ -34,3 +36,19 @@ class UndefType(object):
 
 
 Undef = UndefType()
+
+
+# XXX The default is 1 for now.  In the future we'll invert this logic so
+#     that it's enabled by default.
+has_cextensions = False
+if os.environ.get("STORM_CEXTENSIONS") == "1":
+    try:
+        from storm import cextensions
+        has_cextensions = True
+    except ImportError, e:
+        # XXX Once the logic is inverted and cextensions are enabled by
+        #     default, use the following version so that people may opt
+        #     to use and distribute the pure Python version.
+        #if "cextensions" not in str(e):
+        #    raise
+        raise
