@@ -29,12 +29,22 @@ from storm.variables import (
     Variable, RawStrVariable, UnicodeVariable, LazyValue,
     DateTimeVariable, DateVariable, TimeVariable, TimeDeltaVariable,
     BoolVariable, IntVariable, FloatVariable, DecimalVariable)
-from storm import Undef
+from storm import Undef, has_cextensions
 
 
 # --------------------------------------------------------------------
 # Basic compiler infrastructure
 
+def _when(self, types):
+    """Check Compile.when.  Defined here to ease the work of cextensions."""
+    def decorator(method):
+        for type in types:
+            self._local_dispatch_table[type] = method
+        self._update_cache()
+        return method
+    return decorator
+
+
 class Compile(object):
     """Compiler based on the concept of generic functions."""
 
@@ -45,7 +55,6 @@ class Compile(object):
         self._dispatch_table = {}
         self._precedence = {}
         self._reserved_words = {}
-        self._hash = None
         self._children = WeakKeyDictionary()
         self._parents = []
         if parent:
@@ -65,6 +74,18 @@ class Compile(object):
         for child in self._children:
             child._update_cache()
 
+    def when(self, *types):
+        """Decorator to include a type handler in this compiler.
+
+        Use this as:
+
+        @compile.when(TypeA, TypeB)
+        def compile_type_a_or_b(compile, expr, state):
+            ...
+            return "THE COMPILED SQL STATEMENT"
+        """
+        return _when(self, types)
+
     def add_reserved_words(self, words):
         """Include words to be considered reserved and thus escaped.
 
@@ -91,14 +112,6 @@ class Compile(object):
         """
         return self.__class__(self)
 
-    def when(self, *types):
-        def decorator(method):
-            for type in types:
-                self._local_dispatch_table[type] = method
-            self._update_cache()
-            return method
-        return decorator
-
     def get_precedence(self, type):
         return self._precedence.get(type, MAX_PRECEDENCE)
 
@@ -132,7 +145,7 @@ class Compile(object):
             return "(%s)" % statement
         return statement
 
-    def __call__(self, expr, state=None, join=", ", raw=False, token=False):
+    def __call__(self, expr, state=None, join=u", ", raw=False, token=False):
         """Compile the given expression into a SQL statement.
 
         @param expr: The expression to compile.
@@ -170,7 +183,7 @@ class Compile(object):
                     statement = subexpr
                 elif subexpr_type is tuple or subexpr_type is list:
                     state.precedence = outer_precedence
-                    statement = self(subexpr, state, join, raw)
+                    statement = self(subexpr, state, join, raw, token)
                 else:
                     if token and (subexpr_type is unicode or
                                   subexpr_type is str):
@@ -186,11 +199,25 @@ class Compile(object):
         return statement
 
 
+if has_cextensions:
+    from storm.cextensions import Compile
+
+
 class CompilePython(Compile):
 
     def get_matcher(self, expr):
-        exec "def match(get_column): return bool(%s)" % self(expr)
-        return match
+        state = State()
+        source = self(expr, state)
+        namespace = {}
+        code = ("def closure(parameters, bool):\n"
+                "    [%s] = parameters\n"
+                "    def match(get_column):\n"
+                "        return bool(%s)\n"
+                "    return match" %
+                (",".join("_%d" % i for i in range(len(state.parameters))),
+                 source))
+        exec code in namespace
+        return namespace['closure'](state.parameters, bool)
 
 
 class State(object):
@@ -331,12 +358,18 @@ def compile_none(compile, expr, state):
     return "NULL"
 
 
-@compile_python.when(str, unicode, bool, int, long, float,
-                     datetime, date, time, timedelta, type(None))
+@compile_python.when(str, unicode, int, long, float, type(None))
 def compile_python_builtin(compile, expr, state):
     return repr(expr)
 
 
+@compile_python.when(bool, datetime, date, time, timedelta)
+def compile_python_bool_and_dates(compile, expr, state):
+    index = len(state.parameters)
+    state.parameters.append(expr)
+    return "_%d" % index
+
+
 @compile.when(Variable)
 def compile_variable(compile, variable, state):
     state.parameters.append(variable)
@@ -344,7 +377,9 @@ def compile_variable(compile, variable, state):
 
 @compile_python.when(Variable)
 def compile_python_variable(compile, variable, state):
-    return repr(variable.get())
+    index = len(state.parameters)
+    state.parameters.append(variable.get())
+    return "_%d" % index
 
 
 # --------------------------------------------------------------------
@@ -353,7 +388,7 @@ def compile_python_variable(compile, variable, state):
 MAX_PRECEDENCE = 1000
 
 class Expr(LazyValue):
-    pass
+    __slots__ = ()
 
 @compile_python.when(Expr)
 def compile_python_unsupported(compile, expr, state):
@@ -361,6 +396,7 @@ def compile_python_unsupported(compile, expr, state):
 
 
 class Comparable(object):
+    __slots__ = ()
 
     def __eq__(self, other):
         if other is not None and not isinstance(other, (Expr, Variable)):
@@ -437,11 +473,14 @@ class Comparable(object):
             other = getattr(self, "variable_factory", Variable)(value=other)
         return Mod(self, other)
 
+    def __neg__(self):
+        return Neg(self)
+
     def is_in(self, others):
         if not isinstance(others, Expr):
-            if not others:
-                return None
             others = list(others)
+            if not others:
+                return False
             variable_factory = getattr(self, "variable_factory", Variable)
             for i, other in enumerate(others):
                 if not isinstance(other, (Expr, Variable)):
@@ -461,15 +500,17 @@ class Comparable(object):
 
 
 class ComparableExpr(Expr, Comparable):
-    pass
+    __slots__ = ()
 
 class BinaryExpr(ComparableExpr):
+    __slots__ = ("expr1", "expr2")
 
     def __init__(self, expr1, expr2):
         self.expr1 = expr1
         self.expr2 = expr2
 
 class CompoundExpr(ComparableExpr):
+    __slots__ = ("exprs",)
 
     def __init__(self, *exprs):
         self.exprs = exprs
@@ -508,7 +549,7 @@ def build_tables(compile, tables, default_tables, state):
     # If it's a single element, it's trivial.
     if type(tables) not in (list, tuple) or len(tables) == 1:
         return compile(tables, state, token=True)
- 
+
     # If we have no joins, it's trivial as well.
     for elem in tables:
         if isinstance(elem, JoinExpr):
@@ -567,11 +608,13 @@ def build_tables(compile, tables, default_tables, state):
 
 
 class Select(Expr):
+    __slots__ = ("columns", "where", "tables", "default_tables", "order_by",
+                 "group_by", "limit", "offset", "distinct", "having")
 
     def __init__(self, columns, where=Undef,
                  tables=Undef, default_tables=Undef,
                  order_by=Undef, group_by=Undef,
-                 limit=Undef, offset=Undef, distinct=False):
+                 limit=Undef, offset=Undef, distinct=False, having=Undef):
         self.columns = columns
         self.where = where
         self.tables = tables
@@ -581,6 +624,7 @@ class Select(Expr):
         self.limit = limit
         self.offset = offset
         self.distinct = distinct
+        self.having = having
 
 @compile.when(Select)
 def compile_select(compile, select, state):
@@ -599,6 +643,9 @@ def compile_select(compile, select, state):
     if select.group_by is not Undef:
         tokens.append(" GROUP BY ")
         tokens.append(compile(select.group_by, state, raw=True))
+    if select.having is not Undef:
+        tokens.append(" HAVING ")
+        tokens.append(compile(select.having, state, raw=True))
     if select.order_by is not Undef:
         tokens.append(" ORDER BY ")
         tokens.append(compile(select.order_by, state, raw=True))
@@ -634,6 +681,8 @@ class Insert(Expr):
         key of the table where the row will be inserted.  This is a hint used
         by backends to process the insertion of rows.
     """
+    __slots__ = ("map", "table", "default_table", "primary_columns",
+                 "primary_variables")
 
     def __init__(self, map, table=Undef, default_table=Undef,
                  primary_columns=Undef, primary_variables=Undef):
@@ -657,6 +706,7 @@ def compile_insert(compile, insert, state):
 
 
 class Update(Expr):
+    __slots__ = ("map", "where", "table", "default_table")
 
     def __init__(self, map, where=Undef, table=Undef, default_table=Undef):
         self.map = map
@@ -684,6 +734,7 @@ def compile_update(compile, update, state):
 
 
 class Delete(Expr):
+    __slots__ = ("where", "table", "default_table")
 
     def __init__(self, where=Undef, table=Undef, default_table=Undef):
         self.where = where
@@ -719,6 +770,7 @@ class Column(ComparableExpr):
     @ivar variable_factory: Factory producing C{Variable} instances typed
         according to this column.
     """
+    __slots__ = ("name", "table", "primary", "variable_factory")
 
     def __init__(self, name=Undef, table=Undef, primary=False,
                  variable_factory=None):
@@ -745,7 +797,9 @@ def compile_column(compile, column, state):
 
 @compile_python.when(Column)
 def compile_python_column(compile, column, state):
-    return "get_column(%s)" % repr(column.name)
+    index = len(state.parameters)
+    state.parameters.append(column)
+    return "get_column(_%d)" % index
 
 
 # --------------------------------------------------------------------
@@ -754,6 +808,7 @@ def compile_python_column(compile, column, state):
 class Alias(ComparableExpr):
     """A representation of "AS" alias clauses. e.g., SELECT foo AS bar.
     """
+    __slots__ = ("expr", "name")
 
     auto_counter = 0
 
@@ -781,10 +836,11 @@ def compile_alias(compile, alias, state):
 # From expressions
 
 class FromExpr(Expr):
-    pass
+    __slots__ = ()
 
 
 class Table(FromExpr):
+    __slots__ = ("name",)
 
     def __init__(self, name):
         self.name = name
@@ -795,22 +851,22 @@ def compile_table(compile, table, state):
 
 
 class JoinExpr(FromExpr):
+    __slots__ = ("left", "right", "on")
 
-    left = on = Undef
     oper = "(unknown)"
 
     def __init__(self, arg1, arg2=Undef, on=Undef):
         # http://www.postgresql.org/docs/8.1/interactive/explicit-joins.html
         if arg2 is Undef:
+            self.left = Undef
             self.right = arg1
-            if on is not Undef:
-                self.on = on
+            self.on = on
         elif not isinstance(arg2, Expr) or isinstance(arg2, (FromExpr, Alias)):
             self.left = arg1
             self.right = arg2
-            if on is not Undef:
-                self.on = on
+            self.on = on
         else:
+            self.left = Undef
             self.right = arg1
             self.on = arg2
             if on is not Undef:
@@ -820,14 +876,15 @@ class JoinExpr(FromExpr):
 @compile.when(JoinExpr)
 def compile_join(compile, join, state):
     result = []
-    # Ensure that nested JOINs get parentheses.
-    state.precedence += 0.5
     if join.left is not Undef:
         statement = compile(join.left, state, token=True)
         result.append(statement)
         if state.join_tables is not None:
             state.join_tables.add(statement)
     result.append(join.oper)
+    # Joins are left associative, so ensure joins in the right hand
+    # argument get parentheses.
+    state.precedence += 0.5
     statement = compile(join.right, state, token=True)
     result.append(statement)
     if state.join_tables is not None:
@@ -841,21 +898,27 @@ def compile_join(compile, join, state):
 
 
 class Join(JoinExpr):
+    __slots__ = ()
     oper = "JOIN"
 
 class LeftJoin(JoinExpr):
+    __slots__ = ()
     oper = "LEFT JOIN"
 
 class RightJoin(JoinExpr):
+    __slots__ = ()
     oper = "RIGHT JOIN"
 
 class NaturalJoin(JoinExpr):
+    __slots__ = ()
     oper = "NATURAL JOIN"
 
 class NaturalLeftJoin(JoinExpr):
+    __slots__ = ()
     oper = "NATURAL LEFT JOIN"
 
 class NaturalRightJoin(JoinExpr):
+    __slots__ = ()
     oper = "NATURAL RIGHT JOIN"
 
 
@@ -863,6 +926,7 @@ class NaturalRightJoin(JoinExpr):
 # Operators
 
 class BinaryOper(BinaryExpr):
+    __slots__ = ()
     oper = " (unknown) "
 
 @compile.when(BinaryOper)
@@ -873,6 +937,7 @@ def compile_binary_oper(compile, expr, state):
 
 
 class NonAssocBinaryOper(BinaryOper):
+    __slots__ = ()
     oper = " (unknown) "
 
 @compile.when(NonAssocBinaryOper)
@@ -885,6 +950,7 @@ def compile_non_assoc_binary_oper(compile, expr, state):
 
 
 class CompoundOper(CompoundExpr):
+    __slots__ = ()
     oper = " (unknown) "
 
 @compile.when(CompoundOper)
@@ -897,6 +963,7 @@ def compile_compound_oper(compile, expr, state):
 
 
 class Eq(BinaryOper):
+    __slots__ = ()
     oper = " = "
 
 @compile.when(Eq)
@@ -911,6 +978,7 @@ def compile_eq(compile, eq, state):
 
 
 class Ne(BinaryOper):
+    __slots__ = ()
     oper = " != "
 
 @compile.when(Ne)
@@ -921,25 +989,32 @@ def compile_ne(compile, ne, state):
 
 
 class Gt(BinaryOper):
+    __slots__ = ()
     oper = " > "
 
 class Ge(BinaryOper):
+    __slots__ = ()
     oper = " >= "
 
 class Lt(BinaryOper):
+    __slots__ = ()
     oper = " < "
 
 class Le(BinaryOper):
+    __slots__ = ()
     oper = " <= "
 
 class RShift(BinaryOper):
+    __slots__ = ()
     oper = ">>"
 
 class LShift(BinaryOper):
+    __slots__ = ()
     oper = "<<"
 
 
 class Like(BinaryOper):
+    __slots__ = ("escape", "case_sensitive")
     oper = " LIKE "
 
     def __init__(self, expr1, expr2, escape=Undef, case_sensitive=None):
@@ -961,6 +1036,7 @@ compile_python.when(Like)(compile_python_unsupported)
 
 
 class In(BinaryOper):
+    __slots__ = ()
     oper = " IN "
 
 @compile.when(In)
@@ -977,25 +1053,32 @@ def compile_in(compile, expr, state):
 
 
 class Add(CompoundOper):
+    __slots__ = ()
     oper = "+"
 
 class Sub(NonAssocBinaryOper):
+    __slots__ = ()
     oper = "-"
 
 class Mul(CompoundOper):
+    __slots__ = ()
     oper = "*"
 
 class Div(NonAssocBinaryOper):
+    __slots__ = ()
     oper = "/"
 
 class Mod(NonAssocBinaryOper):
+    __slots__ = ()
     oper = "%"
 
 
 class And(CompoundOper):
+    __slots__ = ()
     oper = " AND "
 
 class Or(CompoundOper):
+    __slots__ = ()
     oper = " OR "
 
 @compile.when(And, Or)
@@ -1007,6 +1090,7 @@ def compile_compound_oper(compile, expr, state):
 # Set expressions.
 
 class SetExpr(Expr):
+    __slots__ = ("exprs", "all", "order_by", "limit", "offset")
     oper = " (unknown) "
 
     def __init__(self, *exprs, **kwargs):
@@ -1071,12 +1155,15 @@ def compile_set_expr(compile, expr, state):
 
 
 class Union(SetExpr):
+    __slots__ = ()
     oper = " UNION "
 
 class Except(SetExpr):
+    __slots__ = ()
     oper = " EXCEPT "
 
 class Intersect(SetExpr):
+    __slots__ = ()
     oper = " INTERSECT "
 
 
@@ -1084,10 +1171,12 @@ class Intersect(SetExpr):
 # Functions
 
 class FuncExpr(ComparableExpr):
+    __slots__ = ()
     name = "(unknown)"
 
 
 class Count(FuncExpr):
+    __slots__ = ("column", "distinct")
     name = "COUNT"
 
     def __init__(self, column=Undef, distinct=False):
@@ -1099,52 +1188,72 @@ class Count(FuncExpr):
 @compile.when(Count)
 def compile_count(compile, count, state):
     if count.column is not Undef:
+        state.push("context", EXPR)
+        column = compile(count.column, state)
+        state.pop()
         if count.distinct:
-            return "COUNT(DISTINCT %s)" % compile(count.column, state)
-        return "COUNT(%s)" % compile(count.column, state)
+            return "COUNT(DISTINCT %s)" % column
+        return "COUNT(%s)" % column
     return "COUNT(*)"
 
 
 class Func(FuncExpr):
+    __slots__ = ("name", "args")
 
     def __init__(self, name, *args):
         self.name = name
         self.args = args
 
 class NamedFunc(FuncExpr):
+    __slots__ = ("args",)
 
     def __init__(self, *args):
         self.args = args
 
 @compile.when(Func, NamedFunc)
 def compile_func(compile, func, state):
-    return "%s(%s)" % (func.name, compile(func.args, state))
+    state.push("context", EXPR)
+    args = compile(func.args, state)
+    state.pop()
+    return "%s(%s)" % (func.name, args)
 
 
 class Max(NamedFunc):
+    __slots__ = ()
     name = "MAX"
 
 class Min(NamedFunc):
+    __slots__ = ()
     name = "MIN"
 
 class Avg(NamedFunc):
+    __slots__ = ()
     name = "AVG"
 
 class Sum(NamedFunc):
+    __slots__ = ()
     name = "SUM"
 
 
 class Lower(NamedFunc):
+    __slots__ = ()
     name = "LOWER"
 
 class Upper(NamedFunc):
+    __slots__ = ()
     name = "UPPER"
 
 
+class Coalesce(NamedFunc):
+    __slots__ = ()
+    name = "COALESCE"
+
+
 # --------------------------------------------------------------------
 # Prefix and suffix expressions
 
 class PrefixExpr(Expr):
+    __slots__ = ("expr",)
     prefix = "(unknown)"
 
     def __init__(self, expr):
@@ -1156,6 +1265,7 @@ def compile_prefix_expr(compile, expr, state):
 
 
 class SuffixExpr(Expr):
+    __slots__ = ("expr",)
     suffix = "(unknown)"
 
     def __init__(self, expr):
@@ -1167,15 +1277,27 @@ def compile_suffix_expr(compile, expr, state):
 
 
 class Not(PrefixExpr):
+    __slots__ = ()
     prefix = "NOT"
 
 class Exists(PrefixExpr):
+    __slots__ = ()
     prefix = "EXISTS"
 
+class Neg(PrefixExpr):
+    __slots__ = ()
+    prefix = "-"
+
+@compile_python.when(Neg)
+def compile_neg_expr(compile, expr, state):
+    return "-%s" % compile(expr.expr, state, raw=True)
+
 class Asc(SuffixExpr):
+    __slots__ = ()
     suffix = "ASC"
 
 class Desc(SuffixExpr):
+    __slots__ = ()
     suffix = "DESC"
 
 
@@ -1187,6 +1309,7 @@ class SQLRaw(str):
 
     This is handled internally by the compiler.
     """
+    __slots__ = ()
 
 
 class SQLToken(str):
@@ -1194,6 +1317,7 @@ class SQLToken(str):
 
     These strings will be quoted, when needed.
     """
+    __slots__ = ()
 
 is_safe_token = re.compile("^[a-zA-Z][a-zA-Z0-9_]*$").match
 
@@ -1209,6 +1333,7 @@ def compile_python_sql_token(compile, expr, state):
 
 
 class SQL(ComparableExpr):
+    __slots__ = ("expr", "params", "tables")
 
     def __init__(self, expr, params=Undef, tables=Undef):
         self.expr = expr
@@ -1244,6 +1369,7 @@ class Sequence(Expr):
           (...)
           id = Int(default=Sequence("my_sequence_name"))
     """
+    __slots__ = ("name",)
 
     def __init__(self, name):
         self.name = name
@@ -1272,26 +1398,28 @@ def compare_columns(columns, values):
 # --------------------------------------------------------------------
 # Auto table
 
-class AutoTable(Expr):
-    """This class will inject an entry in state.auto_tables.
+class AutoTables(Expr):
+    """This class will inject one or more entries in state.auto_tables.
 
     If the constructor is passed replace=True, it will also discard any
     auto_table entries injected by compiling the given expression.
     """
+    __slots__ = ("expr", "tables", "replace")
 
-    def __init__(self, expr, table, replace=False):
+    def __init__(self, expr, tables, replace=False):
+        assert type(tables) in (list, tuple)
         self.expr = expr
-        self.table = table
+        self.tables = tables
         self.replace = replace
 
-@compile.when(AutoTable)
-def compile_auto_table(compile, expr, state):
+@compile.when(AutoTables)
+def compile_auto_tables(compile, expr, state):
     if expr.replace:
         state.push("auto_tables", [])
     statement = compile(expr.expr, state)
     if expr.replace:
         state.pop()
-    state.auto_tables.append(expr.table)
+    state.auto_tables.extend(expr.tables)
     return statement
 
 
