@@ -22,9 +22,13 @@ from datetime import datetime, date, time, timedelta
 from decimal import Decimal
 import cPickle as pickle
 import re
+try:
+    import uuid
+except ImportError:
+    uuid = None
 
 from storm.exceptions import NoneError
-from storm import Undef
+from storm import Undef, has_cextensions
 
 
 __all__ = [
@@ -42,6 +46,7 @@ __all__ = [
     "TimeVariable",
     "TimeDeltaVariable",
     "EnumVariable",
+    "UUIDVariable",
     "PickleVariable",
     "ListVariable",
 ]
@@ -49,6 +54,7 @@ __all__ = [
 
 class LazyValue(object):
     """Marker to be used as a base class on lazily evaluated values."""
+    __slots__ = ()
 
 
 def raise_none_error(column):
@@ -98,12 +104,16 @@ class Variable(object):
     _lazy_value = Undef
     _checkpoint_state = Undef
     _allow_none = True
+    _validator = None
+    _validator_object_factory = None
+    _validator_attribute = None
 
     column = None
     event = None
 
     def __init__(self, value=Undef, value_factory=Undef, from_db=False,
-                 allow_none=True, column=None, event=None):
+                 allow_none=True, column=None, event=None, validator=None,
+                 validator_object_factory=None, validator_attribute=None):
         """
         @param value: The initial value of this variable. The default
             behavior is for the value to stay undefined until it is
@@ -115,6 +125,14 @@ class Variable(object):
             specified.
         @param allow_none: A boolean indicating whether None should be
             allowed to be set as the value of this variable.
+        @param validator: Validation function called whenever trying to
+            set the variable to a non-db value.  The function should
+            look like validator(object, attr, value), where the first and
+            second arguments are the result of validator_object_factory()
+            (or None, if this parameter isn't provided) and the value of
+            validator_attribute, respectively.  When called, the function
+            should raise an error if the value is unacceptable, or return
+            the value to be used in place of the original value otherwise.
         @type column: L{storm.expr.Column}
         @param column: The column that this variable represents. It's
             used for reporting better error messages.
@@ -128,6 +146,10 @@ class Variable(object):
             self.set(value, from_db)
         elif value_factory is not Undef:
             self.set(value_factory(), from_db)
+        if validator is not None:
+            self._validator = validator
+            self._validator_object_factory = validator_object_factory
+            self._validator_attribute = validator_attribute
         self.column = column
         self.event = event
 
@@ -181,8 +203,14 @@ class Variable(object):
 
         if isinstance(value, LazyValue):
             self._lazy_value = value
-            new_value = Undef
+            self._checkpoint_state = new_value = Undef
         else:
+            if not from_db and self._validator is not None:
+                # We use a factory rather than the object itself to prevent
+                # the cycle object => obj_info => variable => object
+                value = self._validator(self._validator_object_factory and
+                                        self._validator_object_factory(),
+                                        self._validator_attribute, value)
             self._lazy_value = Undef
             if value is None:
                 if self._allow_none is False:
@@ -262,15 +290,6 @@ class Variable(object):
         variable.set_state(self.get_state())
         return variable
 
-    def __eq__(self, other):
-        """Equality based on current value, not identity."""
-        return (self.__class__ is other.__class__ and
-                self._value == other._value)
-
-    def __hash__(self):
-        """Hash based on current value, not identity."""
-        return hash(self._value)
-
     def parse_get(self, value, to_db):
         """Convert the internal value to an external value.
 
@@ -300,14 +319,12 @@ class Variable(object):
         return value
 
 
-try:
+if has_cextensions:
     from storm.cextensions import Variable
-except ImportError, e:
-    if "cextensions" not in str(e):
-        raise
 
 
 class BoolVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if not isinstance(value, (int, long, float, Decimal)):
@@ -317,6 +334,7 @@ class BoolVariable(Variable):
 
 
 class IntVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if not isinstance(value, (int, long, float, Decimal)):
@@ -326,6 +344,7 @@ class IntVariable(Variable):
 
 
 class FloatVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if not isinstance(value, (int, long, float, Decimal)):
@@ -335,6 +354,7 @@ class FloatVariable(Variable):
 
 
 class DecimalVariable(Variable):
+    __slots__ = ()
 
     @staticmethod
     def parse_set(value, from_db):
@@ -354,6 +374,7 @@ class DecimalVariable(Variable):
 
 
 class RawStrVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if isinstance(value, buffer):
@@ -365,6 +386,7 @@ class RawStrVariable(Variable):
 
 
 class UnicodeVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if not isinstance(value, unicode):
@@ -374,6 +396,7 @@ class UnicodeVariable(Variable):
 
 
 class DateTimeVariable(Variable):
+    __slots__ = ("_tzinfo",)
 
     def __init__(self, *args, **kwargs):
         self._tzinfo = kwargs.pop("tzinfo", None)
@@ -407,6 +430,7 @@ class DateTimeVariable(Variable):
 
 
 class DateVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if from_db:
@@ -428,6 +452,7 @@ class DateVariable(Variable):
 
 
 class TimeVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if from_db:
@@ -450,6 +475,7 @@ class TimeVariable(Variable):
 
 
 class TimeDeltaVariable(Variable):
+    __slots__ = ()
 
     def parse_set(self, value, from_db):
         if from_db:
@@ -467,7 +493,26 @@ class TimeDeltaVariable(Variable):
             return value
 
 
+class UUIDVariable(Variable):
+    __slots__ = ()
+
+    def parse_set(self, value, from_db):
+        assert uuid is not None, "The uuid module was not found."
+        if from_db and isinstance(value, basestring):
+            value = uuid.UUID(value)
+        elif not isinstance(value, uuid.UUID):
+            raise TypeError("Expected UUID, found %r: %r"
+                            % (type(value), value))
+        return value
+
+    def parse_get(self, value, to_db):
+        if to_db:
+            return str(value)
+        return value
+
+
 class EnumVariable(Variable):
+    __slots__ = ("_get_map", "_set_map")
 
     def __init__(self, get_map, set_map, *args, **kwargs):
         self._get_map = get_map
@@ -491,18 +536,52 @@ class EnumVariable(Variable):
             raise ValueError("Invalid enum value: %s" % repr(value))
 
 
-class PickleVariable(Variable):
+class MutableValueVariable(Variable):
+    """
+    A variable which contains a reference to mutable content. For this kind
+    of variable, we can't simply detect when a modification has been made, so
+    we have to synchronize the content of the variable when the store is
+    flushing current objects, to check if the state has changed.
+    """
+    __slots__ = ("_event_system")
 
     def __init__(self, *args, **kwargs):
+        self._event_system = None
         Variable.__init__(self, *args, **kwargs)
-        if self.event:
-            self.event.hook("flush", self._detect_changes)
+        if self.event is not None:
+            self.event.hook("start-tracking-changes", self._start_tracking)
             self.event.hook("object-deleted", self._detect_changes)
 
+    def _start_tracking(self, obj_info, event_system):
+        self._event_system = event_system
+        self.event.hook("stop-tracking-changes", self._stop_tracking)
+
+    def _stop_tracking(self, obj_info, event_system):
+        event_system.unhook("flush", self._detect_changes)
+        self._event_system = None
+
     def _detect_changes(self, obj_info):
-        if self.get_state() != self._checkpoint_state:
+        if (self._checkpoint_state is not Undef and
+            self.get_state() != self._checkpoint_state):
             self.event.emit("changed", self, None, self._value, False)
 
+    def get(self, default=None, to_db=False):
+        if self._event_system is not None:
+            self._event_system.hook("flush", self._detect_changes)
+        return super(MutableValueVariable, self).get(default, to_db)
+
+    def set(self, value, from_db=False):
+        if self._event_system is not None:
+            if isinstance(value, LazyValue):
+                self._event_system.unhook("flush", self._detect_changes)
+            else:
+                self._event_system.hook("flush", self._detect_changes)
+        super(MutableValueVariable, self).set(value, from_db)
+
+
+class PickleVariable(MutableValueVariable):
+    __slots__ = ()
+
     def parse_set(self, value, from_db):
         if from_db:
             if isinstance(value, buffer):
@@ -524,25 +603,13 @@ class PickleVariable(Variable):
         self._lazy_value = state[0]
         self._value = pickle.loads(state[1])
 
-    def __hash__(self):
-        try:
-            return hash(self._value)
-        except TypeError:
-            return hash(pickle.dumps(self._value, -1))
 
-
-class ListVariable(Variable):
+class ListVariable(MutableValueVariable):
+    __slots__ = ("_item_factory",)
 
     def __init__(self, item_factory, *args, **kwargs):
         self._item_factory = item_factory
-        Variable.__init__(self, *args, **kwargs)
-        if self.event:
-            self.event.hook("flush", self._detect_changes)
-            self.event.hook("object-deleted", self._detect_changes)
-
-    def _detect_changes(self, obj_info):
-        if self.get_state() != self._checkpoint_state:
-            self.event.emit("changed", self, None, self._value, False)
+        MutableValueVariable.__init__(self, *args, **kwargs)
 
     def parse_set(self, value, from_db):
         if from_db:
@@ -555,8 +622,7 @@ class ListVariable(Variable):
     def parse_get(self, value, to_db):
         if to_db:
             item_factory = self._item_factory
-            # XXX This from_db=to_db is dubious. What to do here?
-            return [item_factory(value=val, from_db=to_db) for val in value]
+            return [item_factory(value=val, from_db=False) for val in value]
         else:
             return value
 
@@ -567,9 +633,6 @@ class ListVariable(Variable):
         self._lazy_value = state[0]
         self._value = pickle.loads(state[1])
 
-    def __hash__(self):
-        return hash(pickle.dumps(self._value, -1))
-
 
 def _parse_time(time_str):
     # TODO Add support for timezones.
