@@ -26,14 +26,15 @@ import sys
 import os
 
 from storm.uri import URI
-from storm.expr import Select, Column, Undef, SQLToken, SQLRaw, Count, Alias
+from storm.expr import Select, Column, SQLToken, SQLRaw, Count, Alias
 from storm.variables import (Variable, PickleVariable, RawStrVariable,
                              DecimalVariable, DateTimeVariable, DateVariable,
                              TimeVariable, TimeDeltaVariable)
 from storm.database import *
 from storm.event import EventSystem
 from storm.exceptions import (
-    DatabaseModuleError, DisconnectionError, OperationalError)
+    DatabaseError, DatabaseModuleError, DisconnectionError, Error,
+    OperationalError)
 
 from tests.databases.proxy import ProxyTCPServer
 from tests.helper import MakePath
@@ -377,6 +378,23 @@ class DatabaseTest(object):
         result.from_database = self.from_database
         self.assertEquals(iter(result).next(), (2, 3))
 
+    def test_rowcount_insert(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute(
+            "INSERT INTO test VALUES (999, '999')")
+        self.assertEquals(result.rowcount, 1)
+
+    def test_rowcount_delete(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute("DELETE FROM test")
+        self.assertEquals(result.rowcount, 2)
+
+    def test_rowcount_update(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute(
+            "UPDATE test SET title='whatever'")
+        self.assertEquals(result.rowcount, 2)
+
 
 class UnsupportedDatabaseTest(object):
     
@@ -515,6 +533,61 @@ class DatabaseDisconnectionTest(object):
         self.proxy.restart()
         self.assertRaises(DisconnectionError, self.connection.commit)
 
+    def test_wb_catch_already_disconnected_on_rollback(self):
+        """Connection.rollback() swallows disconnection errors.
+
+        If the connection is being used outside of Storm's control,
+        then it is possible that Storm won't see the disconnection.
+        It should be able to recover from this situation though.
+        """
+        result = self.connection.execute("SELECT 1")
+        self.assertTrue(result.get_one())
+        self.proxy.restart()
+        # Perform an action that should result in a disconnection.
+        try:
+            cursor = self.connection._raw_connection.cursor()
+            cursor.execute("SELECT 1")
+            cursor.fetchone()
+        except Error, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not caught.")
+
+        # Make sure our raw connection's rollback() raises a disconnection
+        # error when called.
+        try:
+            self.connection._raw_connection.rollback()
+        except Error, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not raised.")
+
+        # Our rollback() will catch and swallow that disconnection error,
+        # though.
+        self.connection.rollback()
+
+    def test_wb_catch_already_disconnected(self):
+        """Storm detects connections that have already been disconnected.
+
+        If the connection is being used outside of Storm's control,
+        then it is possible that Storm won't see the disconnection.
+        It should be able to recover from this situation though.
+        """
+        result = self.connection.execute("SELECT 1")
+        self.assertTrue(result.get_one())
+        self.proxy.restart()
+        # Perform an action that should result in a disconnection.
+        try:
+            cursor = self.connection._raw_connection.cursor()
+            cursor.execute("SELECT 1")
+            cursor.fetchone()
+        except DatabaseError, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not caught.")
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "SELECT 1")
+
     def test_connection_stays_disconnected_in_transaction(self):
         """Test that connection does not immediately reconnect."""
         result = self.connection.execute("SELECT 1")
