@@ -1316,6 +1316,20 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, "UPPER(func1())")
         self.assertEquals(state.parameters, [])
 
+    def test_coalesce(self):
+        expr = Coalesce(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1())")
+        self.assertEquals(state.parameters, [])
+
+    def test_coalesce_with_many_arguments(self):
+        expr = Coalesce(Func1(), Func2(), None)
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1(), func2(), NULL)")
+        self.assertEquals(state.parameters, [])
+
     def test_not(self):
         expr = Not(Func1())
         state = State()
@@ -1330,6 +1344,19 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, "EXISTS func1()")
         self.assertEquals(state.parameters, [])
 
+    def test_neg(self):
+        expr = Neg(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
+        expr = -Func1()
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
     def test_asc(self):
         expr = Asc(Func1())
         state = State()
@@ -1937,8 +1964,10 @@ class CompilePythonTest(TestHelper):
 
     def test_compile_sequence(self):
         expr = [elem1, Variable(1), (Variable(2), None)]
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "elem1, 1, 2, None")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "elem1, _0, _1, None")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_compile_invalid(self):
         self.assertRaises(CompileError, compile_python, object())
@@ -1965,8 +1994,10 @@ class CompilePythonTest(TestHelper):
         self.assertEquals(py_expr, "1L")
 
     def test_bool(self):
-        py_expr = compile_python(True)
-        self.assertEquals(py_expr, "True")
+        state = State()
+        py_expr = compile_python(True, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [True])
 
     def test_float(self):
         py_expr = compile_python(1.1)
@@ -1974,23 +2005,31 @@ class CompilePythonTest(TestHelper):
 
     def test_datetime(self):
         dt = datetime(1977, 5, 4, 12, 34)
-        py_expr = compile_python(dt)
-        self.assertEquals(py_expr, repr(dt))
+        state = State()
+        py_expr = compile_python(dt, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [dt])
 
     def test_date(self):
         d = date(1977, 5, 4)
-        py_expr = compile_python(d)
-        self.assertEquals(py_expr, repr(d))
+        state = State()
+        py_expr = compile_python(d, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [d])
 
     def test_time(self):
         t = time(12, 34)
-        py_expr = compile_python(t)
-        self.assertEquals(py_expr, repr(t))
+        state = State()
+        py_expr = compile_python(t, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [t])
 
     def test_timedelta(self):
         td = timedelta(days=1, seconds=2, microseconds=3)
-        py_expr = compile_python(td)
-        self.assertEquals(py_expr, repr(td))
+        state = State()
+        py_expr = compile_python(td, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [td])
 
     def test_none(self):
         py_expr = compile_python(None)
@@ -1998,63 +2037,87 @@ class CompilePythonTest(TestHelper):
 
     def test_column(self):
         expr = Column(column1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_column_table(self):
         expr = Column(column1, table1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_variable(self):
         expr = Variable("value")
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "'value'")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, ["value"])
 
     def test_eq(self):
         expr = Eq(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 == 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 == _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ne(self):
         expr = Ne(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 != 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 != _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_gt(self):
         expr = Gt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 > 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 > _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ge(self):
         expr = Ge(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 >= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 >= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lt(self):
         expr = Lt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 < 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 < _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_le(self):
         expr = Le(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 <= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 <= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lshift(self):
         expr = LShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1<<2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0<<_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_rshift(self):
         expr = RShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1>>2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0>>_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_in(self):
         expr = In(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 in (2,)")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 in (_1,)")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_and(self):
         expr = And(elem1, elem2, And(elem3, elem4))
@@ -2071,6 +2134,11 @@ class CompilePythonTest(TestHelper):
         py_expr = compile_python(expr)
         self.assertEquals(py_expr, "elem1+elem2+elem3+elem4")
 
+    def test_neg(self):
+        expr = Neg(elem1)
+        py_expr = compile_python(expr)
+        self.assertEquals(py_expr, "-elem1")
+
     def test_sub(self):
         expr = Sub(elem1, Sub(elem2, elem3))
         py_expr = compile_python(expr)
@@ -2109,8 +2177,20 @@ class CompilePythonTest(TestHelper):
 
         match = compile_python.get_matcher((col1 > 10) & (col2 < 10))
 
-        self.assertTrue(match({"column1": 15, "column2": 5}.get))
-        self.assertFalse(match({"column1": 5, "column2": 15}.get))
+        self.assertTrue(match({col1: 15, col2: 5}.get))
+        self.assertFalse(match({col1: 5, col2: 15}.get))
+
+    def test_match_bad_repr(self):
+        """The get_matcher() works for expressions containing values
+        whose repr is not valid Python syntax."""
+        class BadRepr(object):
+            def __repr__(self):
+                return "$Not a valid Python expression$"
+
+        value = BadRepr()
+        col1 = Column(column1)
+        match = compile_python.get_matcher(col1 == Variable(value))
+        self.assertTrue(match({col1: value}.get))
 
 
 class LazyValueExprTest(TestHelper):
