@@ -21,6 +21,12 @@
 from datetime import datetime, date, time, timedelta
 from decimal import Decimal
 import cPickle as pickle
+import gc
+import weakref
+try:
+    import uuid
+except ImportError:
+    uuid = None
 
 from storm.exceptions import NoneError
 from storm.variables import *
@@ -91,6 +97,18 @@ class VariableTest(TestHelper):
         variable.set(marker, from_db=True)
         self.assertEquals(variable.sets, [(marker, False), (marker, True)])
 
+    def test_set_leak(self):
+        """When a variable is checkpointed, the value must not leak."""
+        variable = Variable()
+        m = Marker()
+        m_ref = weakref.ref(m)
+        variable.set(m)
+        variable.checkpoint()
+        variable.set(LazyValue())
+        del m
+        gc.collect()
+        self.assertIdentical(m_ref(), None)
+
     def test_get(self):
         variable = CustomVariable()
         variable.set(marker)
@@ -763,6 +781,47 @@ class ParseIntervalTest(TestHelper):
             self.fail("ValueError not raised")
 
 
+class UUIDVariableTest(TestHelper):
+
+    def is_supported(self):
+        return uuid is not None
+
+    def test_get_set(self):
+        value = uuid.UUID("{0609f76b-878f-4546-baf5-c1b135e8de72}")
+
+        variable = UUIDVariable()
+
+        variable.set(value)
+        self.assertEquals(variable.get(), value)
+        self.assertEquals(
+            variable.get(to_db=True), "0609f76b-878f-4546-baf5-c1b135e8de72")
+
+        self.assertRaises(TypeError, variable.set, marker)
+        self.assertRaises(TypeError, variable.set,
+                          "0609f76b-878f-4546-baf5-c1b135e8de72")
+        self.assertRaises(TypeError, variable.set,
+                          u"0609f76b-878f-4546-baf5-c1b135e8de72")
+
+    def test_get_set_from_database(self):
+        value = uuid.UUID("{0609f76b-878f-4546-baf5-c1b135e8de72}")
+
+        variable = UUIDVariable()
+
+        # Strings and UUID objects are accepted from the database.
+        variable.set(value, from_db=True)
+        self.assertEquals(variable.get(), value)
+        variable.set("0609f76b-878f-4546-baf5-c1b135e8de72", from_db=True)
+        self.assertEquals(variable.get(), value)
+        variable.set(u"0609f76b-878f-4546-baf5-c1b135e8de72", from_db=True)
+        self.assertEquals(variable.get(), value)
+
+        # Some other representations for UUID values.
+        variable.set("{0609f76b-878f-4546-baf5-c1b135e8de72}", from_db=True)
+        self.assertEquals(variable.get(), value)
+        variable.set("0609f76b878f4546baf5c1b135e8de72", from_db=True)
+        self.assertEquals(variable.get(), value)
+
+
 class PickleVariableTest(TestHelper):
 
     def test_get_set(self):
@@ -828,20 +887,26 @@ class PickleVariableTest(TestHelper):
 class ListVariableTest(TestHelper):
 
     def test_get_set(self):
-        l = [1, 2]
-        l_dump = pickle.dumps(l, -1)
+        # Enumeration variables are used as items so that database
+        # side and python side values can be distinguished.
+        get_map = {1: "a", 2: "b", 3: "c"}
+        set_map = {"a": 1, "b": 2, "c": 3}
+        item_factory = VariableFactory(
+            EnumVariable, get_map=get_map, set_map=set_map)
 
-        variable = ListVariable(IntVariable)
+        l = ["a", "b"]
+        l_dump = pickle.dumps(l, -1)
+        l_vars = [item_factory(value=x) for x in l]
+
+        variable = ListVariable(item_factory)
 
         variable.set(l)
         self.assertEquals(variable.get(), l)
-        self.assertVariablesEqual(variable.get(to_db=True),
-                                  [IntVariable(1), IntVariable(2)])
+        self.assertVariablesEqual(variable.get(to_db=True), l_vars)
 
-        variable.set([1.1, 2.2], from_db=True)
+        variable.set([1, 2], from_db=True)
         self.assertEquals(variable.get(), l)
-        self.assertVariablesEqual(variable.get(to_db=True),
-                                  [IntVariable(1), IntVariable(2)])
+        self.assertVariablesEqual(variable.get(to_db=True), l_vars)
 
         self.assertEquals(variable.get_state(), (Undef, l_dump))
 
@@ -849,8 +914,8 @@ class ListVariableTest(TestHelper):
         variable.set_state((Undef, l_dump))
         self.assertEquals(variable.get(), l)
 
-        variable.get().append(3)
-        self.assertEquals(variable.get(), [1, 2, 3])
+        variable.get().append("c")
+        self.assertEquals(variable.get(), ["a", "b", "c"])
 
     def test_list_events(self):
         event = EventSystem(marker)
